@@ -55,12 +55,37 @@ static GArray* gtk2hs_finalizers;
 
 gboolean gtk2hs_run_finalizers(gpointer data);
 
+#if defined( WIN32 ) && GLIB_CHECK_VERSION(2,32,0)
+static GRecMutex recursive_mutex;
+
+void imp_rec_lock() {
+    g_rec_mutex_lock(&recursive_mutex);
+}
+
+void imp_rec_unlock() {
+    g_rec_mutex_unlock(&recursive_mutex);
+}
+#endif
+
 /* Initialize the default _fmode on WIN32. */
 void gtk2hs_initialise (void) {
 #if defined( WIN32 ) && defined( GTK2HS_SET_FMODE_BINARY )
 	/* Some Windows GTK binraries (current Fedora MinGW ones) do */
 	/* not open files in binary mode.  This is a work around.    */
-    _fmode = _O_BINARY;
+    HANDLE handle = LoadLibrary("MSVCRT.dll");
+    if(!handle) { 
+        fprintf(stderr, "Warning: failed to load MSVCRT.dll, ");
+        fprintf(stderr, "binary mode was not set!\n");
+        return;
+    }
+    
+    int *_fmode_ptr = GetProcAddress(handle, "_fmode");
+    if(!_fmode_ptr) {
+        fprintf(stderr, "Warning: failed to load address of _fmode from MSVCRT.dll, ");
+        fprintf(stderr, "binary mode was not set!\n");
+        return;
+    }
+    *_fmode_ptr = _O_BINARY;
 #endif
 }
 
@@ -79,6 +104,12 @@ void gtk2hs_threads_initialise (void) {
 #else
     g_static_mutex_init(&gtk2hs_finalizer_mutex);
 #endif
+
+#if defined( WIN32 ) && GLIB_CHECK_VERSION(2,32,0)
+    g_rec_mutex_init(&recursive_mutex);
+    
+    gdk_threads_set_lock_functions(imp_rec_lock, imp_rec_unlock);
+#endif
     g_thread_init(NULL);
     gdk_threads_init();
 
